package com.mobilepcmonitor.sample.cloud;

import java.net.InetAddress;

import com.mmsoftdesign.cloud.Service;
import com.mmsoftdesign.cloud.handlers.CommandReceivedHandler;
import com.mmsoftdesign.cloud.handlers.DetailsRequestHandler;
import com.mmsoftdesign.cloud.handlers.ExceptionOccurredHandler;
import com.mmsoftdesign.cloud.handlers.InputValueChangedHandler;
import com.mmsoftdesign.cloud.handlers.PageCommandReceivedHandler;
import com.mmsoftdesign.cloud.handlers.PageRequestHandler;
import com.mmsoftdesign.cloud.model.CommandItem;
import com.mmsoftdesign.cloud.model.Date;
import com.mmsoftdesign.cloud.model.DateInputItem;
import com.mmsoftdesign.cloud.model.DateTime;
import com.mmsoftdesign.cloud.model.DateTimeInputItem;
import com.mmsoftdesign.cloud.model.Group;
import com.mmsoftdesign.cloud.model.Groups;
import com.mmsoftdesign.cloud.model.NotificationPriority;
import com.mmsoftdesign.cloud.model.NumberInputItem;
import com.mmsoftdesign.cloud.model.PageItem;
import com.mmsoftdesign.cloud.model.PickListInputItem;
import com.mmsoftdesign.cloud.model.PickListItem;
import com.mmsoftdesign.cloud.model.PickListItems;
import com.mmsoftdesign.cloud.model.SimpleItem;
import com.mmsoftdesign.cloud.model.TextInputItem;
import com.mmsoftdesign.cloud.model.Time;
import com.mmsoftdesign.cloud.model.TimeInputItem;

public class Cloud {

	public static void main(String[] args) {
		new Cloud();
	}

	private int numberOfNotificationsSent;
	private int detailsInvocationCount;

	private static final int COMMAND_SEND_TEST_NOTIFICATION = 1;
	private static final int COMMAND_CLEAR_NOTIFICATIONS_COUNT = 2;
	private static final int COMMAND_CLEAR_INVOCATIONS_COUNT = 3;
	private static final int COMMAND_STOP_INSTANCE_MONITORING = 4;

	private static final int PAGE_FIRST = 1;
	private static final int PAGE_SECOND = 2;

	private static final String INPUT_TEXT = "inputText";
	private static final String INPUT_NUMBER = "inputNumber";
	private static final String INPUT_DATE = "inputDate";
	private static final String INPUT_TIME = "inputTime";
	private static final String INPUT_DATETIME = "inputDateTime";
	private static final String INPUT_PICKLIST = "inputPickList";

	private String textValue;
	private Integer numberValue;
	private Date dateValue;
	private Time timeValue;
	private DateTime dateTimeValue;
	private PickListItem pickListValue;

	private static PickListItems items = new PickListItems();
	static {
		items.add(new PickListItem(1, "Option 1", "Description 1"));
		items.add(new PickListItem(2, "Option 2", "Description 2"));
		items.add(new PickListItem(3, "Option 3", "Description 3"));
	}

	private final InputValueChangedHandler inputValueChangedHandler = new InputValueChangedHandler() {

		@Override
		public void onTimeInputValueChanged(String inputId, Time inputValue) {
			if (inputId.equals(INPUT_TIME)) {
				timeValue = inputValue;
			}
		}

		@Override
		public void onTextInputValueChanged(String inputId, String inputValue) {
			if (inputId.equals(INPUT_TEXT)) {
				textValue = inputValue;
			}

		}

		@Override
		public void onPickListInputValueChanged(String inputId,
				int pickListItemId) {
			if (inputId.equals(INPUT_PICKLIST)) {
				pickListValue = items.findListItemWithId(pickListItemId);
			}

		}

		@Override
		public void onNumericInputValueChanged(String inputId, int inputValue) {
			if (inputId.equals(INPUT_NUMBER)) {
				numberValue = inputValue;
			}

		}

		@Override
		public void onDateTimeInputValueChanged(String inputId,
				DateTime inputValue) {
			if (inputId.equals(INPUT_DATETIME)) {
				dateTimeValue = inputValue;
			}
		}

		@Override
		public void onDateInputValueChanged(String inputId, Date inputValue) {
			if (inputId.equals(INPUT_DATE)) {
				dateValue = inputValue;
			}
		}
	};

	public Cloud() {
		// Subscribing to events
		Service.getInstance().setDetailsRequestHandler(
				new DetailsRequestHandler() {
					public void onDetailsRequest() {
						onDetailsRequestHandler();
					}
				});
		Service.getInstance().setPageRequestHandler(new PageRequestHandler() {
			public void onPageRequest(int pageId, String mobileDeviceIdentifier) {
				onPageRequestHandler(pageId, mobileDeviceIdentifier);
			}
		});
		Service.getInstance().setCommandReceivedHandler(
				new CommandReceivedHandler() {
					public void onCommandReceived(int commandId,
							String mobileDeviceIdentifier) {
						onCommandReceivedHandler(commandId,
								mobileDeviceIdentifier);
					}
				});
		Service.getInstance().setPageCommandReceivedHandler(
				new PageCommandReceivedHandler() {
					public void onPageCommandReceived(int pageId,
							int commandId, String mobileDeviceIdentifier) {
						onPageCommandReceivedHandler(pageId, commandId,
								mobileDeviceIdentifier);
					}
				});
		Service.getInstance().setExceptionOccurredHandler(
				new ExceptionOccurredHandler() {
					public void onExceptionOccurred(Exception exception) {
						onExceptionOccurredHandler(exception);
					}
				});

		Service.getInstance().setInputValueChangedHandler(
				inputValueChangedHandler);
		String hostName = "Unknown";
		try {
			hostName = InetAddress.getLocalHost().getHostName();
		} catch (Exception e) {
		}
		// Configuring the cloud instance
		Service.getInstance().configure("Test Cloud", "Cloud",
				"Running on " + hostName, false /*
												 * set to true if you want to be
												 * notified when instance is
												 * offline
												 */);

		String username = "Your Pulseway username";
		String password = "Your Pulseway password";
		String apiKey = "Your Pulseway API Key"; // Email us at
													// support@pulseway.com
													// with your Pulseway
													// username to receive your
													// API Key
		
		// Starting the instance monitoring
		Service.getInstance().start(username, password, apiKey);
	}

	// Called when the instance details are requested by the mobile application
	private void onDetailsRequestHandler() {
		detailsInvocationCount++;

		Groups details = new Groups();

		Group firstGroup = new Group("Values");
		firstGroup.getItems().add(
				new SimpleItem(Integer.toString(numberOfNotificationsSent),
						"Notifications Sent"));
		firstGroup.getItems().add(
				new SimpleItem(Integer.toString(detailsInvocationCount),
						"Details Request Invocation Count"));
		details.add(firstGroup);

		Group secondGroup = new Group("Commands");
		secondGroup.getItems().add(
				new CommandItem(COMMAND_SEND_TEST_NOTIFICATION,
						"Send a test notification"));
		secondGroup.getItems().add(
				new CommandItem(COMMAND_CLEAR_NOTIFICATIONS_COUNT,
						"Clear Notifications Count"));
		secondGroup.getItems().add(
				new CommandItem(COMMAND_CLEAR_INVOCATIONS_COUNT,
						"Clear Details Request Count"));
		secondGroup.getItems().add(
				new CommandItem(COMMAND_STOP_INSTANCE_MONITORING, "Stop",
						"Stop Monitoring this Instance"));
		details.add(secondGroup);

		Group thirdGroup = new Group("Pages");
		thirdGroup.getItems().add(
				new PageItem(PAGE_FIRST, "Page 1", "This is a page"));
		details.add(thirdGroup);

		Group fourthGroup = new Group("Inputs");
		fourthGroup.getItems().add(
				new TextInputItem(INPUT_TEXT, "Text",
						textValue == null ? "No value" : "Value:" + textValue,
						textValue));

		fourthGroup.getItems().add(
				new NumberInputItem(INPUT_NUMBER, "Number",
						numberValue == null ? "No value " : "Value:"
								+ Integer.toString(numberValue), numberValue));

		fourthGroup.getItems().add(
				new DateInputItem(INPUT_DATE, "Date",
						dateValue == null ? "No value" : "Value:" + dateValue,
						dateValue));
		fourthGroup.getItems().add(
				new TimeInputItem(INPUT_TIME, "Time",
						timeValue == null ? "No value" : "Value:" + timeValue,
						timeValue));
		fourthGroup.getItems().add(
				new DateTimeInputItem(INPUT_DATETIME, "Date/Time",
						dateTimeValue == null ? "No value" : "Value:"
								+ dateTimeValue, dateTimeValue));

		fourthGroup
				.getItems()
				.add(new PickListInputItem(INPUT_PICKLIST, "PickList",
						pickListValue == null ? "No value" : "Value:"
								+ pickListValue.getName(), items, pickListValue));

		details.add(fourthGroup);

		Service.getInstance().setDetails(details);
	}

	// Called when the a page is requested by the mobile application
	private void onPageRequestHandler(int pageId, String mobileDeviceIdentifier) {
		Groups details = new Groups();

		if (pageId == PAGE_FIRST) {
			Group firstGroup = new Group("First Page Items");
			firstGroup.getItems().add(new SimpleItem("1", "Simple Value"));
			firstGroup.getItems().add(
					new CommandItem(COMMAND_SEND_TEST_NOTIFICATION,
							"Send a test notification"));
			firstGroup.getItems()
					.add(new PageItem(PAGE_SECOND, "Page 2",
							"This is another page"));
			details.add(firstGroup);
		} else if (pageId == PAGE_SECOND) {
			Group firstGroup = new Group("Second Page Items");
			firstGroup.getItems().add(new SimpleItem("2", "Simple Value"));
			firstGroup.getItems().add(
					new CommandItem(COMMAND_SEND_TEST_NOTIFICATION,
							"Send a test notification"));
			details.add(firstGroup);
		}

		Service.getInstance().setPageDetails(pageId, details);
	}

	// Called when a command is executed on the mobile application
	private void onCommandReceivedHandler(int commandId,
			String mobileDeviceIdentifier) {
		if (commandId == COMMAND_SEND_TEST_NOTIFICATION) {
			numberOfNotificationsSent++;
			// Send a notification to all registered devices on the account
			Service.getInstance().sendNotificationToAllDevices(
					"This is a test notification from the Cloud API",
					NotificationPriority.NORMAL);
		} else if (commandId == COMMAND_CLEAR_NOTIFICATIONS_COUNT) {
			numberOfNotificationsSent = 0;
		} else if (commandId == COMMAND_CLEAR_INVOCATIONS_COUNT) {
			detailsInvocationCount = 0;
		} else if (commandId == COMMAND_STOP_INSTANCE_MONITORING) {
			// Stop the instance monitoring
			Service.getInstance().stop();
		}
	}

	// Called when a command is executed from a page on the mobile application
	private void onPageCommandReceivedHandler(int pageId, int commandId,
			String mobileDeviceIdentifier) {
		if (commandId == COMMAND_SEND_TEST_NOTIFICATION) {
			numberOfNotificationsSent++;
			// Send a notification to all registered devices on the account

			if (pageId == PAGE_FIRST) {
				Service.getInstance()
						.sendNotificationToAllDevices(
								"This is a test notification from the 1st Page Cloud API",
								NotificationPriority.CRITICAL);
			} else if (pageId == PAGE_SECOND) {
				Service.getInstance()
						.sendNotificationToAllDevices(
								"This is another notification from the 2nd Page Cloud API",
								NotificationPriority.CRITICAL);
			}
		}
	}

	// Called when an exception occurred in the Cloud API
	private void onExceptionOccurredHandler(Exception ex) {
		// Get more info about the error
		ex.printStackTrace();
	}
}
