package com.mmsoftdesign.cloud.handlers;

import com.mmsoftdesign.cloud.model.Date;
import com.mmsoftdesign.cloud.model.DateTime;
import com.mmsoftdesign.cloud.model.Time;

/**
 * Handler for input value change events.
 * 
 * @author calin.perebiceanu
 * 
 */
public interface InputValueChangedHandler {
	/**
	 * Called when a text input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param inputValue
	 *            Item input value
	 */
	public abstract void onTextInputValueChanged(String inputId, String inputValue);

	/**
	 * Called when a numeric input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param inputValue
	 *            Item input value
	 */
	public abstract void onNumericInputValueChanged(String inputId, int inputValue);

	/**
	 * Called when a date input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param inputValue
	 *            Item input value
	 */
	public abstract void onDateInputValueChanged(String inputId, Date inputValue);

	/**
	 * Called when a time input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param inputValue
	 *            Item input value
	 */
	public abstract void onTimeInputValueChanged(String inputId, Time inputValue);

	/**
	 * Called when a date and time input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param inputValue
	 *            Item input value
	 */
	public abstract void onDateTimeInputValueChanged(String inputId,
			DateTime inputValue);

	/**
	 * Called when a pick list input item changes
	 * 
	 *@param inputId
	 *            Item input Id
	 *@param pickListItemId
	 *            Item input value
	 */
	public abstract void onPickListInputValueChanged(String inputId,
			int pickListItemId);
}
